@@ -53,6 +53,7 @@ from .base import (
 )
 from .exceptions import (
     CaptionReadNoCaptions, CaptionReadSyntaxError, InvalidInputError,
+    CaptionReadTimingError
 )
 from .geometry import Layout, Alignment, Padding, Size
 
@@ -86,7 +87,7 @@ class SAMIReader(BaseReader):
 
         content, doc_styles, doc_langs = (
             self._get_sami_parser_class()().feed(content))
-        sami_soup = self._get_xml_parser_class()(content)
+        sami_soup = self._get_xml_parser_class()(content, features="lxml")
 
         # Get the global layout that applies to all <p> tags
         global_layout = self._build_layout(doc_styles.get('p', {}))
@@ -191,7 +192,11 @@ class SAMIReader(BaseReader):
         milliseconds = 0
 
         for p in sami_soup.select(f'p[lang|={language}]'):
-            milliseconds = int(float(p.parent['start']))
+            start_str = p.parent.get('start')
+            if not start_str:
+                raise CaptionReadTimingError(
+                    f"Missing start time on the following line: {p.parent}.")
+            milliseconds = int(float(start_str))
             start = milliseconds * 1000
             end = 0
 
