@@ -1,15 +1,27 @@
 import collections
-import unicodedata
 
-from ..base import CaptionList, Caption, CaptionNode
+from ..base import Caption, CaptionList, CaptionNode
 from ..geometry import (
-    UnitEnum, Size, Layout, Point, Alignment,
-    VerticalAlignmentEnum, HorizontalAlignmentEnum
+    Alignment,
+    HorizontalAlignmentEnum,
+    Layout,
+    Point,
+    Size,
+    UnitEnum,
+    VerticalAlignmentEnum,
 )
 from .constants import (
-    PAC_BYTES_TO_POSITIONING_MAP, COMMANDS, PAC_TAB_OFFSET_COMMANDS,
-    MICROSECONDS_PER_CODEWORD, BACKGROUND_COLOR_CODES,
-    MID_ROW_CODES, EXTENDED_CHARS, SPECIAL_CHARS
+    BACKGROUND_COLOR_CODES,
+    COMMANDS,
+    EXTENDED_CHARS,
+    ITALICS_COMMANDS,
+    MICROSECONDS_PER_CODEWORD,
+    MID_ROW_CODES,
+    PAC_BYTES_TO_POSITIONING_MAP,
+    PAC_TAB_OFFSET_COMMANDS,
+    PLAIN_TEXT_COMMANDS,
+    STYLE_SETTING_COMMANDS,
+    UNDERLINE_COMMANDS,
 )
 
 PopOnCue = collections.namedtuple("PopOnCue", "buffer, start, end")
@@ -32,9 +44,7 @@ class PreCaption:
         self.layout_info = None
 
     def to_real_caption(self):
-        return Caption(
-            self.start, self.end, self.nodes, self.style, self.layout_info
-        )
+        return Caption(self.start, self.end, self.nodes, self.style, self.layout_info)
 
 
 class TimingCorrectingCaptionList(list):
@@ -46,6 +56,7 @@ class TimingCorrectingCaptionList(list):
 
     Also, doesn't allow Nones or empty captions
     """
+
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
         self._last_batch = ()
@@ -99,9 +110,10 @@ class TimingCorrectingCaptionList(list):
 
         new_caption = new_captions[0]
 
-        if batch and (batch[-1].end == 0
-                      or new_caption.start - batch[-1].end
-                      < 5 * MICROSECONDS_PER_CODEWORD + 1):
+        if batch and (
+            batch[-1].end == 0
+            or new_caption.start - batch[-1].end < 5 * MICROSECONDS_PER_CODEWORD + 1
+        ):
             for caption in batch:
                 caption.end = new_caption.start
 
@@ -110,6 +122,7 @@ class NotifyingDict(dict):
     """Dictionary-like object, that treats one key as 'active',
     and notifies observers if the active key changed
     """
+
     # Need an unhashable object as initial value for the active key.
     # That way we're sure this was never a key in the dict.
     _guard = {}
@@ -125,7 +138,7 @@ class NotifyingDict(dict):
         :param key: any hashable object
         """
         if key not in self:
-            raise ValueError('No such key present')
+            raise ValueError("No such key present")
 
         # Notify observers of the change
         if key != self.active_key:
@@ -137,7 +150,7 @@ class NotifyingDict(dict):
     def get_active(self):
         """Returns the value corresponding to the active key"""
         if self.active_key is self._guard:
-            raise KeyError('No active key set')
+            raise KeyError("No active key set")
 
         return self[self.active_key]
 
@@ -151,13 +164,14 @@ class NotifyingDict(dict):
             arguments
         """
         if not callable(observer):
-            raise TypeError('The observer should be callable')
+            raise TypeError("The observer should be callable")
 
         self.observers.append(observer)
 
 
 class CaptionCreator:
     """Creates and maintains a collection of Captions"""
+
     def __init__(self):
         self._collection = TimingCorrectingCaptionList()
 
@@ -227,35 +241,41 @@ class CaptionCreator:
 
             # handle line breaks
             elif instruction.is_explicit_break():
-                caption.nodes.append(CaptionNode.create_break(
-                    layout_info=_get_layout_from_tuple(instruction.position)
-                ))
+                caption.nodes.append(
+                    CaptionNode.create_break(
+                        layout_info=_get_layout_from_tuple(instruction.position)
+                    )
+                )
 
             # handle open italics
             elif instruction.sets_italics_on():
                 caption.nodes.append(
                     CaptionNode.create_style(
-                        True, {'italics': True},
-                        layout_info=_get_layout_from_tuple(
-                            instruction.position
-                        ))
+                        True,
+                        {"italics": True},
+                        layout_info=_get_layout_from_tuple(instruction.position),
+                    )
                 )
 
             # handle clone italics
             elif instruction.sets_italics_off():
                 caption.nodes.append(
                     CaptionNode.create_style(
-                        False, {'italics': True},
-                        layout_info=_get_layout_from_tuple(
-                            instruction.position)
-                    ))
+                        False,
+                        {"italics": True},
+                        layout_info=_get_layout_from_tuple(instruction.position),
+                    )
+                )
 
             # handle text
             elif instruction.is_text_node():
                 layout_info = _get_layout_from_tuple(instruction.position)
                 caption.nodes.append(
                     CaptionNode.create_text(
-                        instruction.text, layout_info=layout_info),
+                        text=instruction.text,
+                        layout_info=layout_info,
+                        position=instruction.position,
+                    )
                 )
                 caption.layout_info = layout_info
 
@@ -276,6 +296,7 @@ class InstructionNodeCreator:
     """Creates _InstructionNode instances from characters and commands, storing
     them internally
     """
+
     def __init__(self, collection=None, position_tracker=None):
         """
         :param collection: an optional collection of nodes
@@ -288,8 +309,9 @@ class InstructionNodeCreator:
         else:
             self._collection = collection
 
-        self.last_style = None
-
+        self.last_style = (
+            None  # can be italic on or italic off as we only support italics
+        )
         self._position_tracer = position_tracker
 
     def is_empty(self):
@@ -307,8 +329,11 @@ class InstructionNodeCreator:
         current_position = self._position_tracer.get_current_position()
 
         # get or create a usable node
-        if (self._collection and self._collection[-1].is_text_node()
-                and not self._position_tracer.is_repositioning_required()):
+        if (
+            self._collection
+            and self._collection[-1].is_text_node()
+            and not self._position_tracer.is_repositioning_required()
+        ):
             node = self._collection[-1]
         else:
             # create first node
@@ -317,19 +342,21 @@ class InstructionNodeCreator:
 
         # handle a simple line break
         if self._position_tracer.is_linebreak_required():
-            # must insert a line break here
-            self._collection.append(_InstructionNode.create_break(
-                position=current_position))
+            self._collection.append(
+                _InstructionNode.create_break(position=current_position)
+            )
+            self._position_tracer.acknowledge_linebreak_consumed()
             node = _InstructionNode.create_text(current_position)
             self._collection.append(node)
-            self._position_tracer.acknowledge_linebreak_consumed()
+            if self._position_tracer.is_repositioning_required():
+                # it means we have a reposition command which was not followed by
+                # any text, so we just ignore it and break
+                self._position_tracer.acknowledge_position_changed()
 
         # handle completely new positioning
         elif self._position_tracer.is_repositioning_required():
             self._collection.append(
-                _InstructionNode.create_repositioning_command(
-                    current_position
-                )
+                _InstructionNode.create_repositioning_command(current_position)
             )
             node = _InstructionNode.create_text(current_position)
             self._collection.append(node)
@@ -337,20 +364,29 @@ class InstructionNodeCreator:
 
         node.add_chars(*chars)
 
-    def interpret_command(self, command, previous_is_pac_or_tab=False):
+    @staticmethod
+    def get_style_for_command(command):
+        if command in ITALICS_COMMANDS:
+            return "italic"
+        elif command in UNDERLINE_COMMANDS:
+            return "underline"
+        else:
+            # as we only check STYLE_SETTING_COMMANDS,
+            # only remaining possibility is plain text
+            return "plaintext"
+
+    def interpret_command(self, command, next_command=None):
         """Given a command determines whether to turn italics on or off,
         or to set the positioning
 
         This is mostly used to convert from the legacy-style commands
 
         :type command: str
-        :type previous_is_pac_or_tab: previous command code is for a PAC command
         or a PAC_TAB_OFFSET_COMMANDS
+        :type next_command: the command that follows next
         """
         self._update_positioning(command)
 
-        text = COMMANDS.get(command, '')
-
         if command == "94a1":
             self.handle_backspace("94a1")
 
@@ -360,60 +396,114 @@ class InstructionNodeCreator:
             # which will be deleted when the code is applied.
             # ex: 2080 97ad 94a1
             if (
-                    self._collection[-1].is_text_node() and
-                    self._collection[-1].text[-1].isspace()
+                len(self._collection) > 0
+                and self._collection[-1].is_text_node()
+                and self._collection[-1].text[-1].isspace()
             ):
                 self._collection[-1].text = self._collection[-1].text[:-1]
 
-        if 'italic' in text:
-            if 'end' not in text:
-                self._collection.append(
-                    _InstructionNode.create_italics_style(
-                        self._position_tracer.get_current_position())
-                )
-                self.last_style = "italics on"
-            else:
-                self._collection.append(
-                    _InstructionNode.create_italics_style(
-                        self._position_tracer.get_current_position(),
-                        turn_on=False
+        if command in STYLE_SETTING_COMMANDS:
+            current_position = self._position_tracer.get_current_position()
+            # which style is command setting
+            command_style = self.get_style_for_command(command)
+            if command_style == "italic":
+                if self.last_style is None or self.last_style == "italics off":
+                    #  if we don't have any style yet, or we have a closed italics tag
+                    #  it should open italic tag
+                    #  if break is required, break then add style tag
+                    if self._position_tracer.is_linebreak_required():
+                        self._collection.append(
+                            _InstructionNode.create_break(position=current_position)
+                        )
+                        self._position_tracer.acknowledge_linebreak_consumed()
+                    self._collection.append(
+                        _InstructionNode.create_italics_style(current_position)
                     )
-                )
-                self.last_style = "italics off"
-
-        # mid row code that is not first code on the line
-        # (previous node is not a break node)
-        if command in MID_ROW_CODES and not previous_is_pac_or_tab:
-            if self.last_style == "italics off":
-                self.add_chars(' ')
+                    self.last_style = "italics on"
             else:
-                for node in self._collection[::-1]:
-                    if node.is_text_node() and node.text:
-                        node.text += ' '
-                        break
+                # command sets a different style (underline, plain)
+                # so we need to close italics if we have an open italics tag
+                # otherwise we ignore it
+                # if break is required, add style tag then break
+                if self.last_style == "italics on":
+                    self._collection.append(
+                        _InstructionNode.create_italics_style(
+                            self._position_tracer.get_current_position(), turn_on=False
+                        )
+                    )
+                    self.last_style = "italics off"
+                    if self._position_tracer.is_linebreak_required():
+                        self._collection.append(
+                            _InstructionNode.create_break(position=current_position)
+                        )
+                        self._position_tracer.acknowledge_linebreak_consumed()
+
+        #  handle mid-row codes that follows a text node
+        #  don't add space if the next command adds one of
+        #  ['.', '!', '?', ',']
+        punctuation = ["ae", "a1", "bf", "2c"]
+        next_is_punctuation = next_command and next_command[:2] in punctuation
+        prev_text_node = self.get_previous_text_node()
+        prev_node_is_break = prev_text_node is not None and any(
+            x.is_explicit_break()
+            for x in self._collection[self._collection.index(prev_text_node):]
+        )
+        if (
+            command in MID_ROW_CODES
+            and prev_text_node
+            and not prev_node_is_break
+            and not prev_text_node.text[-1].isspace()
+            and command not in PAC_TAB_OFFSET_COMMANDS
+            and not next_is_punctuation
+        ):
+            if self.last_style == "italics off":
+                # need to open italics tag, add a space
+                # to the beginning of the next text node
+                self.add_chars(" ")
+            else:
+                # italics on
+                # need to close italics tag, add a space
+                # to the end of the previous text node
+                prev_text_node.text = prev_text_node.text + " "
 
     def _update_positioning(self, command):
         """Sets the positioning information to use for the next nodes
 
         :type command: str
         """
+        is_offset = False
         if command in PAC_TAB_OFFSET_COMMANDS:
-            tab_offset = PAC_TAB_OFFSET_COMMANDS[command]
             prev_positioning = self._position_tracer.default
-            positioning = (prev_positioning[0],
-                           prev_positioning[1] + tab_offset)
+            tab_offset = PAC_TAB_OFFSET_COMMANDS[command]
+            positioning = (prev_positioning[0], prev_positioning[1] + tab_offset)
+            is_offset = True
         else:
             first, second = command[:2], command[2:]
-
             try:
+                # is PAC
                 positioning = PAC_BYTES_TO_POSITIONING_MAP[first][second]
             except KeyError:
+                # if not PAC or OFFSET we're not changing position
                 return
-        self._position_tracer.update_positioning(positioning)
+        offset_after_break = is_offset and self.has_break_before(self._collection)
+        if not offset_after_break:
+            # Tab offsets after line breaks will be ignored to avoid repositioning
+            self._position_tracer.update_positioning(positioning)
 
     def __iter__(self):
         return iter(_format_italics(self._collection))
 
+    @staticmethod
+    def has_break_before(collection):
+        if len(collection) == 0:
+            return False
+        for element in collection[::-1]:
+            if element._type == 0:
+                return False
+            elif element._type == 1:
+                return True
+        return False
+
     @classmethod
     def from_list(cls, stash_list, position_tracker):
         """Having received a list of instances of this class, creates a new
@@ -438,7 +528,7 @@ class InstructionNodeCreator:
             # use space to separate the stashes, but don't add final space
             if idx < len(stash_list) - 1:
                 try:
-                    instance._collection[-1].add_chars(' ')
+                    instance._collection[-1].add_chars(" ")
                 except AttributeError:
                     pass
 
@@ -456,9 +546,8 @@ class InstructionNodeCreator:
             return
         last_char = node.text[-1]
         delete_previous_condition = (
-            (word in EXTENDED_CHARS and last_char not in EXTENDED_CHARS.values()) or
-            word == "94a1"
-        )
+            word in EXTENDED_CHARS and last_char not in EXTENDED_CHARS.values()
+        ) or word == "94a1"
         # in case extended char, perform backspace
         # only if the previous character in not also extended
         if delete_previous_condition:
@@ -490,10 +579,10 @@ def _get_layout_from_tuple(position_tuple):
     horizontal = Size(80 * column / 32.0 + 10, UnitEnum.PERCENT)
     # Vertical safe area between 5% and 95%
     vertical = Size(90 * (row - 1) / 15.0 + 5, UnitEnum.PERCENT)
-    return Layout(origin=Point(horizontal, vertical),
-                  alignment=Alignment(HorizontalAlignmentEnum.LEFT,
-                                      VerticalAlignmentEnum.TOP)
-                  )
+    return Layout(
+        origin=Point(horizontal, vertical),
+        alignment=Alignment(HorizontalAlignmentEnum.LEFT, VerticalAlignmentEnum.TOP),
+    )
 
 
 class _InstructionNode:
@@ -503,6 +592,7 @@ class _InstructionNode:
     These nodes will be aggregated into a RepresentableNode, which will then
     be easily converted to a CaptionNode.
     """
+
     TEXT = 0
     BREAK = 1
     ITALICS_ON = 2
@@ -527,9 +617,9 @@ class _InstructionNode:
         :return:
         """
         if self.text is None:
-            self.text = ''
+            self.text = ""
 
-        self.text += ''.join(args)
+        self.text += "".join(args)
 
     def is_text_node(self):
         """
@@ -579,7 +669,7 @@ class _InstructionNode:
 
     def get_text(self):
         """A little legacy code."""
-        return ' '.join(self.text.split())
+        return " ".join(self.text.split())
 
     @classmethod
     def create_break(cls, position):
@@ -604,7 +694,7 @@ class _InstructionNode:
 
         :rtype: _InstructionNode
         """
-        return cls(''.join(chars), position=position)
+        return cls("".join(chars), position=position)
 
     @classmethod
     def create_italics_style(cls, position, turn_on=True):
@@ -619,8 +709,7 @@ class _InstructionNode:
         :rtype: _InstructionNode
         """
         return cls(
-            position=position,
-            type_=cls.ITALICS_ON if turn_on else cls.ITALICS_OFF
+            position=position, type_=cls.ITALICS_ON if turn_on else cls.ITALICS_OFF
         )
 
     @classmethod
@@ -632,19 +721,19 @@ class _InstructionNode:
         """
         return cls(type_=cls.CHANGE_POSITION, position=position)
 
-    def __repr__(self):         # pragma: no cover
+    def __repr__(self):  # pragma: no cover
         if self._type == self.BREAK:
-            extra = 'BR'
+            extra = "BR"
         elif self._type == self.TEXT:
             extra = f'"{self.text}"'
         elif self._type in (self.ITALICS_ON, self.ITALICS_OFF):
-            extra = 'italics {}'.format(
-                'on' if self._type == self.ITALICS_ON else 'off'
+            extra = "italics {}".format(
+                "on" if self._type == self.ITALICS_ON else "off"
             )
         else:
-            extra = 'change position'
+            extra = "change position"
 
-        return f'<INode: {extra} >'
+        return f"<INode: {extra} >"
 
 
 def _format_italics(collection):
@@ -683,9 +772,27 @@ def _format_italics(collection):
     # removes pairs of italics nodes that don't do anything noticeable
     new_collection = _remove_noop_italics(new_collection)
 
+    # remove spaces to the end of the lines
+    new_collection = _remove_spaces_at_end_of_the_line(new_collection)
+
     return new_collection
 
 
+def _remove_spaces_at_end_of_the_line(collection):
+    for idx, node in enumerate(collection):
+        if (
+            idx > 0
+            and node._type == _InstructionNode.BREAK
+            and collection[idx - 1].is_text_node()
+            and collection[idx - 1].text
+        ):
+            collection[idx - 1].text = collection[idx - 1].text.rstrip()
+    # handle last node
+    if collection[-1].is_text_node():
+        collection[-1].text = collection[-1].text.rstrip()
+    return collection
+
+
 def _remove_noop_on_off_italics(collection):
     """Return an equivalent list to `collection`. It removes the italics node
      pairs that don't surround text nodes, if those nodes are in the order:
@@ -792,8 +899,9 @@ def _skip_empty_text_nodes(collection):
     :type collection: list[_InstructionNode]
     :rtype: list[_InstructionNode]
     """
-    return [node for node in collection
-            if not (node.is_text_node() and node.is_empty())]
+    return [
+        node for node in collection if not (node.is_text_node() and node.is_empty())
+    ]
 
 
 def _skip_redundant_italics_nodes(collection):
@@ -811,7 +919,8 @@ def _skip_redundant_italics_nodes(collection):
         if node.is_italics_node():
             if state is None:
                 state = node.sets_italics_on()
-                new_collection.append(node)
+                if node.sets_italics_on():
+                    new_collection.append(node)
                 continue
             # skip the nodes that are like the previous
             if node.sets_italics_on() is state:
@@ -825,19 +934,19 @@ def _skip_redundant_italics_nodes(collection):
 
 def _close_italics_before_repositioning(collection):
     """Make sure that for every opened italic node, there's a corresponding
-     closing node.
+    closing node.
 
-     Will insert a closing italic node, before each repositioning node
+    Will insert a closing italic node, before each repositioning node
 
-     :type collection: list[_InstructionNode]
-     :rtype: list[_InstructionNode]
+    :type collection: list[_InstructionNode]
+    :rtype: list[_InstructionNode]
     """
     new_collection = []
 
     italics_on = False
     last_italics_on_node = None
 
-    for idx, node in enumerate(collection):
+    for node in collection:
         if node.is_italics_node() and node.sets_italics_on():
             italics_on = True
             last_italics_on_node = node
@@ -849,17 +958,16 @@ def _close_italics_before_repositioning(collection):
                 _InstructionNode.create_italics_style(
                     # The position info of this new node should be the same
                     position=last_italics_on_node.position,
-                    turn_on=False
+                    turn_on=False,
                 )
             )
             new_collection.append(node)
             # Append an italics opening node after the positioning change
             new_collection.append(
-                _InstructionNode.create_italics_style(
-                    position=node.position
-                )
+                _InstructionNode.create_italics_style(position=node.position)
             )
             continue
+
         new_collection.append(node)
 
     return new_collection
@@ -886,8 +994,7 @@ def _ensure_final_italics_node_closes(collection):
     if italics_on:
         new_collection.append(
             _InstructionNode.create_italics_style(
-                position=last_italics_on_node.position,
-                turn_on=False
+                position=last_italics_on_node.position, turn_on=False
             )
         )
     return new_collection
